### Kakao Partner SDK Beta (Rx)

Beta 서비스 중에는 기능의 변경이 발생하거나 일시적으로 서비스 접근이 불가할 수 있습니다. [변경내역](#변경-이력)

#### [Kakao SDK V2](https://developers.kakao.com/docs/android/v2) 에 부가적으로 다음과 같은 파트너 특화 기능들을 제공합니다.

* Phase(dev, sandbox, cbt, production) 설정
* UserApi 에 파트너 특화 메소드
* TalkApi 에 파트너 특화 메소드

Reactive-X Kakao SDK for Android 는 Kotlin 기반으로 RxJava 라이브러리를 사용해 재사용성 향상, 비동기 처리 과정 개선과 함께 보다 간결한 인터페이스를 구현한 새로운 버전입니다.

#### 설치

##### Kakao App 설정

[Kakao Developers](https://developers.kakao.com/) 에서 앱을 만들고 Android 플랫폼을 설정합니다. (native-app-key 를 얻을수 있습니다.)

> [Android 로 시작하기](https://developers.kakao.com/docs/android/getting-started)


##### Gradle 설정

Root project 의 build 파일에 repository 설정 부분에 아래와 같이 Kakao 의 repository 를 추가합니다.

```gradle
allprojects {
    repositories {
        google()
        jcenter()
        maven { url 'http://devrepo.kakao.com:8088/nexus/content/groups/public/' }
    }
}
```

사용할 앱의 build 파일에 아래와 같이 파트너 모듈을 import 합니다.

```gradle
dependencies {
  implementation "com.kakao.sdk:partner-rx:2.0.0-beta01"
}
```

##### Dependencies

RxKakaoSDK 는 아래의 라이브러리들을 사용합니다.

- [RxJava2](https://github.com/ReactiveX/RxJava/tree/2.x)
- [Gson](https://github.com/google/gson)
- [Retrofit](https://github.com/square/retrofit)

#### Implementation Guide

- [SDK 초기화](#sdk-초기화)
- [OAuthToken 가져오기](#oauthtoken-가져오기)
- [Token based API 호출하기](#token-based-api-호출하기)
- [App-key based API 호출하기](#appkey-based-api-호출하기)


##### SDK 초기화

SDK를 사용하기 위해서는 native app key 초기화가 필요합니다. ([Kakao App 설정](#Kakao App 설정) 참조)

추가적으로 서버 Phase 를 설정할 수 있습니다. 이때 AppKey는 각 phase 에 해당하는 native app key 로 설정하셔야 합니다.

```kotlin

KakaoSdk.init(
    context = applicationContext,
    appKey = "{put your native app key here}",
    hosts = ServerHosts.withPhase(currentPhase)
)
```


##### OAuthToken 가져오기

먼저 사용자는 Kakao API 를 호출하기 위해 액세스 토큰을 가져와야합니다. 액세스 토큰은 OAuth 2.0 사양에 따라 발행됩니다.

1. 카카오 계정 인증
2. 사용자 동의 (필요하지 않은 경우 건너 뛰기)
3. 인증 코드를 가져오고 (리디렉션을 통해) 액세스 토큰을 발행합니다 (POST API를 통해)

아래에 나열한 두가지 방법이 있습니다.

> 1.  브라우져로 카카오계정 로그인 (Default browser)
> 2.  KakaoTalk 으로 로그인

<a name="브라우져로-카카오계정-로그인-default-browser-"></a>

###### 브라우져로 카카오계정 로그인 (Default browser)

카카오 안드로이드 SDK v2 는 웹뷰를 사용하지 않고 기본 브라우저를 (Custom Tabs) 사용하여 로그인을 진행하기 때문에 아래와 같은 스킴 설정을 *AndroidManifest.xml*에 해주어야 합니다.

```xml
<application>
  ...
  <activity android:name="com.kakao.sdk.auth.AuthCodeCustomTabsActivity">
      <intent-filter>
          <action android:name="android.intent.action.VIEW" />

          <category android:name="android.intent.category.DEFAULT" />
          <category android:name="android.intent.category.BROWSABLE" />

          <data
              android:host="oauth"
              android:scheme="kakao{your-native-app-key-here}" />
      </intent-filter>
  </activity>
  ...
</application>
```

아래와 같이 authorization code 를 발급받고 이어서 OAuthToken 을 발급받을 수 있습니다.

```kotlin

private val disposables: CompositeDisposable = CompositeDisposable()

fun loginButtonClicked() {
  AuthCodeClient.rx.authorize(context)
    .observeOn(Schedulers.io())
    .flatMap { authCode -> AuthApiClient.rx.issueAccessToken(authCode) }
    .subscribe({}, { error -> /* 에러 처리 */ }).addTo(disposables)
}

override fun onDestroy() {
  disposables.dispose() // 메모리 누수를 막기 위해 onDestroy() 에서 반드시 dispose() 호출
  super.onDestroy()
}
```

<a name="kakaotalk-으로-로그인"></a>

###### KakaoTalk 으로 로그인

```kotlin
...

fun loginButtonClicked() {
  AuthCodeClient.rx.authorizeWithTalk(context)
    .observeOne(Schedulers.io())
    .flatMap { authCode -> AuthApiClient.rx.issueAccessToken(authCode) }
    .subscribe({}, { error -> /* 에러 처리 */}).addTo(disposables)
}

override fun onDestroy() {
  disposables.dispose()
  super.onDestroy()
}
```

##### Token based API 호출하기

위 단계에서 access token 을 성공적으로 가져오면 token based API를 호출할 수 있습니다.
다음은 현재 KakaoSDK 에서 지원되는 API 세트 입니다.

- UserApi
- TalkApi
- StoryApi

파트너 SDK 는 UserApiClient 와 TalkApiClient 에 Kotlin extension 으로 확장된 API 들을 제공합니다.

UserApiClient 에 있는 meForPartner()를 호출하는 예제 입니다.

```kotlin
...

private val disposables: CompositeDisposable = CompositeDisposable()

func userMe() {
    val disposed = UserApiClient.rx.meForPartner()
      .subscribe ({ user -> /* 유저 응답 사용 */}, { error -> /* 에러 처리 */})
      .addTo(disposables)
}

...

override fun onDestroy() {
  disposables.dispose(); // 메모리 누수를 막기 위해 onDestroy() 에서 반드시 dispose() 호출
  super.onDestroy()
}
```

##### Appkey based API 호출하기

- KakaoLink

KakaoLink API 는 토큰 기반 API 가 아니기 때문에 로그인 없이 SDK 초기화 이후에 사용할 수 있습니다.

아래는 커스텀 템플릿으로 KakaoLink 메시지를 전송하는 예입니다.

```kotlin
...

func customLink() {
    LinkClient.rx.customTemplate(context, 637)
        doOnSuccess { intent ->
            context.startActivity(intent)
        }
    		.subscribe({}, {error -> /* 에러 처리 */})
        .addTo(disposables)
}

...

override fun onDestroy() {
  disposables.dispose(); // 메모리 누수를 막기 위해 onDestroy() 에서 반드시 dispose() 호출
  super.onDestr0y()
}
```

#### SDK Feature

##### Token 자동갱신

토큰만료 에러시 토큰은 자동갱신됩니다.

##### 동적동의(Dynamic User Agreement)

임의의 사용자의 동의가 필요한 경우 (가입 시 유저가 동의하지 않았거나, 앱설정에 이용중 선택으로 설정한 경우) 직접 유저에게 해당 정보 제공 동의를 받아야 합니다. 이를 [동적동의](https://developers.kakao.com/docs/ios/user-management#동적동의) 라고 합니다. 간단하게 동적동의를 구현할 수 있는 operator 를 제공합니다.

```kotlin

private val disposables: CompositeDisposable = CompositeDisposable()
...

fun talkFriends() {
    TalkApiClient.rx.friends()
        .retryWhen(TokenBasedApiInterceptor.instance.handleInvalidScope(context))
        .subscribe({ friends -> /* 응답 사용 */}, { error -> /* 에러 처리 */})
        .addTo(disposables)
}

...

override fun onDestroy() {
  disposables.dispose() // 메모리 누수를 막기 위해 onDestroy() 에서 반드시 dispose() 호출
  super.onDestroy()
}
```


#### Documentation

Docs 는 [Dokka](https://github.com/Kotlin/dokka) 에 의해 생성되며 결과 문서 페이지는 [KakaoDevelopers]() 에서 제공됩니다.

<a name="변경 이력"></a>

#### 변경 이력

| 변경날짜   | 변경이력         | 버전         |
| :--------- | :--------------- | :----------- |
| 2019.11.25 | 베타 서비스 개시 | 2.0.0-beta01 |

